@@ -14,11 +14,14 @@ from passmanager import Backend
 
 from tables import operator_to_lambda, modules, type_to_suffix
 from tables import builtin_constructors, pytype_to_ctype_table
+from tables import pythran_ward
 from typing import Types
 from syntax import PythranSyntaxError
 
 from openmp import OMPDirective
 
+from math import isnan
+
 import cStringIO
 import unparse
 import metadata
@@ -32,13 +35,12 @@ class Python(Backend):
     >>> node = ast.parse("print 'hello world'")
     >>> pm = passmanager.PassManager('test')
     >>> print pm.dump(Python, node)
-    <BLANKLINE>
     print 'hello world'
     '''
 
     def __init__(self):
         self.result = ''
-        Backend.__init__(self)
+        super(Python, self).__init__()
 
     def visit(self, node):
         output = cStringIO.StringIO()
@@ -52,13 +54,20 @@ def templatize(node, types, default_types=None):
     if types:
         return Template(
                 ["typename {0} {1}".format(
-                    t, "= const {0}".format(d) if d else "")
+                    t, "= {0}".format(d) if d else "")
                     for t, d in zip(types, default_types)],
                 node)
     else:
         return node
 
 
+def strip_exp(s):
+    if s.startswith('(') and s.endswith(')'):
+        return s[1:-1]
+    else:
+        return s
+
+
 class Cxx(Backend):
     '''
     Produces a C++ representation of the AST.
@@ -70,11 +79,10 @@ class Cxx(Backend):
     >>> for l in r: print l
     #include <pythran/pythran.h>
     #include <pythran/pythran_gmp.h>
-    namespace __test
+    namespace __pythran_test
     {
       print(core::string("hello world"));
     }
-    <BLANKLINE>
     '''
 
     # recover previous generator state
@@ -87,27 +95,22 @@ class Cxx(Backend):
         self.declarations = list()
         self.definitions = list()
         self.break_handler = list()
-        self.result = list()
-        Backend.__init__(self,
+        self.result = None
+        super(Cxx, self).__init__(
                 GlobalDeclarations, BoundedExpressions, Types, ArgumentEffects)
 
     # mod
     def visit_Module(self, node):
         # build all types
-        self.local_functions = set()
-        self.local_declarations = list()
-        headers = [Include(h)
-                for h in ("pythran/pythran.h", "pythran/pythran_gmp.h")]
-        # remove top-level strings
-        body = [self.visit(n)
-                for n in node.body if not isinstance(n, ast.Expr)]
+        headers = map(Include, ("pythran/pythran.h", "pythran/pythran_gmp.h"))
 
-        assert not self.local_declarations
-        self.result = headers + [
-                Namespace("__{0}".format(self.passmanager.module_name),
-                    body
-                    + self.declarations
-                    + self.definitions)]
+        # remove top-level strings
+        fbody = (n for n in node.body if not isinstance(n, ast.Expr))
+        body = map(self.visit, fbody)
+
+        nsbody = body + self.declarations + self.definitions
+        ns = Namespace(pythran_ward + self.passmanager.module_name, nsbody)
+        self.result = headers + [ns]
 
     # openmp processing
     def process_omp_attachements(self, node, stmt, index=None):
@@ -118,10 +121,10 @@ class Cxx(Backend):
                 # special hook for default for index scope
                 if isinstance(node, ast.For):
                     target = node.target
-                    if ('for' in directive.s
-                            and 'default' not in directive.s
-                            and all(x.id != target.id for x in directive.deps)
-                            ):
+                    hasfor = 'for' in directive.s
+                    nodefault = 'default' not in directive.s
+                    noindexref = all(x.id != target.id for x in directive.deps)
+                    if hasfor and nodefault and noindexref:
                         directive.s += ' private({})'
                         directive.deps.append(ast.Name(target.id, ast.Param()))
                 directives.append(directive)
@@ -143,39 +146,43 @@ class Cxx(Backend):
 
             def __init__(self):
                 self.cache = dict()
+                self.rcache = dict()
                 self.mapping = dict()
 
             def __call__(self, node):
                 if node not in self.mapping:
                     t = node.generate(self)
-                    self.mapping[node] = len(self.mapping)
-                    self.cache[node] = t
+                    if t in self.rcache:
+                        self.mapping[node] = self.mapping[self.rcache[t]]
+                        self.cache[node] = self.cache[self.rcache[t]]
+                    else:
+                        self.rcache[t] = node
+                        self.mapping[node] = len(self.mapping)
+                        self.cache[node] = t
                 return CachedTypeVisitor.CachedType(
                         "__type{0}".format(self.mapping[node]))
 
             def typedefs(self):
-                l = sorted(self.mapping.items(),
-                        key=lambda x: x[1])
+                l = sorted(self.mapping.items(), key=lambda x: x[1])
                 L = list()
+                visited = set()  # the same value must not be typedefed twice
                 for k, v in l:
-                    L.append(Typedef(
-                        Value(self.cache[k], "__type{0}".format(v))))
+                    if v not in visited:
+                        typename = "__type" + str(v)
+                        L.append(Typedef(Value(self.cache[k], typename)))
+                        visited.add(v)
                 return L
 
         # prepare context and visit function body
         fargs = node.args.args
 
-        local_functions = {k for k in self.local_functions}
-
         formal_args = [arg.id for arg in fargs]
-        formal_types = ["argument_type{0}".format(i)
-                for i in xrange(len(formal_args))]
+        formal_types = ["argument_type" + str(i) for i in xrange(len(fargs))]
 
-        ldecls = {sym.id: sym
-                for sym in self.passmanager.gather(LocalDeclarations, node)}
+        ldecls = self.passmanager.gather(LocalDeclarations, node)
+        ldecls = {sym.id: sym for sym in ldecls}  # more convenient as a dict
 
-        self.local_declarations.append(set(ldecls.iterkeys()))
-        self.local_declarations[-1].update(formal_args)
+        self.local_declarations = set(ldecls.iterkeys()).union(formal_args)
         self.extra_declarations = []
 
         ldecls = set(ldecls.itervalues())
@@ -185,7 +192,10 @@ class Cxx(Backend):
                 for (v, k) in
                 enumerate(self.passmanager.gather(YieldPoints, node))}
 
-        operator_body = [self.visit(n) for n in node.body]
+        # gather body dump
+        operator_body = map(self.visit, node.body)
+
+        # compute arg dump
         default_arg_values = (
                 [None] * (len(node.args.args) - len(node.args.defaults))
                 + [self.visit(n) for n in node.args.defaults])
@@ -193,8 +203,7 @@ class Cxx(Backend):
                 [None] * (len(node.args.args) - len(node.args.defaults))
                 + [self.types[n] for n in node.args.defaults])
 
-        self.local_declarations.pop()
-
+        # compute type dump
         result_type = self.types[node][0]
 
         callable_type = Typedef(Value("void", "callable"))
@@ -245,6 +254,10 @@ class Cxx(Backend):
                         FunctionDeclaration(Value("", next_name), []),
                         Block([]))]
             if formal_types:
+                #if all parameters have a default value, we don't need default
+                # constructor
+                if default_arg_values and all(default_arg_values):
+                    next_constructors = list()
                 next_constructors.append(FunctionBody(
                     make_function_declaration("",
                         next_name, formal_types, formal_args,
@@ -262,6 +275,7 @@ class Cxx(Backend):
                     FunctionBody(
                         FunctionDeclaration(Value("void", "operator++"), []),
                         Block([Statement("next()")])),
+
                     FunctionBody(
                         FunctionDeclaration(
                             Value("typename {0}::result_type".format(
@@ -279,7 +293,7 @@ class Cxx(Backend):
                             []),
                         Block([Statement("next()"),
                             ReturnStatement(
-                                "generator_iterator<{0}>(this)".format(
+                                "generator_iterator<{0}>(*this)".format(
                                     next_name))])),
                     FunctionBody(
                         FunctionDeclaration(
@@ -376,7 +390,7 @@ class Cxx(Backend):
 
             operator_declaration = [
                     templatize(
-                        make_function_declaration(
+                        make_const_function_declaration(
                             instanciated_next_name,
                             "operator()",
                             formal_types,
@@ -385,7 +399,7 @@ class Cxx(Backend):
                         formal_types,
                         default_arg_types),
                     EmptyStatement()]
-            operator_signature = make_function_declaration(
+            operator_signature = make_const_function_declaration(
                     instanciated_next_name,
                     "{0}::operator()".format(node.name),
                     formal_types,
@@ -471,7 +485,6 @@ class Cxx(Backend):
         self.declarations.append(topstruct)
         self.definitions.append(operator_definition)
 
-        self.local_functions = local_functions
         return EmptyStatement()
 
     def visit_Return(self, node):
@@ -509,7 +522,10 @@ class Cxx(Backend):
         targets = [self.visit(t) for t in node.targets]
         alltargets = "= ".join(targets)
         if any(metadata.get(t, metadata.LocalVariable) for t in node.targets):
-            alltargets = "auto {0}".format(alltargets)
+            alltargets = ("typename "
+                    "assignable<decltype({1})>::type {0}".format(
+                        alltargets, value)
+                    )
         stmt = Assign(alltargets, value)
         return self.process_omp_attachements(node, stmt)
 
@@ -547,18 +563,20 @@ class Cxx(Backend):
         local_iter = "__iter{0}".format(len(self.break_handler))
         local_target = "__target{0}".format(len(self.break_handler))
 
-        local_iter_decl = Assignable(DeclType(Val(iter)))
+        local_iter_decl = Assignable(DeclType(iter))
         local_target_decl = NamedType("{0}::iterator".format(local_iter_decl))
         if self.yields:
             self.extra_declarations.append((local_iter, local_iter_decl,))
             self.extra_declarations.append((local_target, local_target_decl,))
             local_target_decl = ""
             local_iter_decl = ""
-        target_decl = ("auto"
-                if metadata.get(node.target, metadata.LocalVariable)
-                else "")
+            target_decl = ""
+        else:
+            target_decl = ("auto"
+                    if metadata.get(node.target, metadata.LocalVariable)
+                    else "")
 
-        body = [self.visit(n) for n in node.body]
+        loop_body = [self.visit(n) for n in node.body]
 
         self.break_handler.pop()
 
@@ -570,26 +588,26 @@ class Cxx(Backend):
                     directive.s += ' shared({})'
                     directive.deps.append(ast.Name(local_iter, ast.Param()))
 
-        stmts = [
-                Statement("{0} {1} = {2}".format(
-                    local_iter_decl,
-                    local_iter, iter)
-                    ),
-                For("{0} {1} = {2}.begin()".format(
+        prelude = Statement("{0} {1} = {2}".format(
+            local_iter_decl, local_iter, iter)
+            )
+        loop_body_prelude = Statement(
+                "{0} {1}= *{2}".format(
+                    target_decl,
+                    target,
+                    local_target)
+                )
+        loop = For(
+                "{0} {1} = {2}.begin()".format(
                     local_target_decl,
                     local_target,
                     local_iter),
-                    "{0} < {1}.end()".format(
-                        local_target,
-                        local_iter),
-                    "++{0}".format(local_target),
-                    Block(
-                        [Statement("{0} {1}= *{2}".format(
-                            target_decl,
-                            target,
-                            local_target))]
-                        + body))
-                ]
+                "{0} < {1}.end()".format(
+                    local_target,
+                    local_iter),
+                "++{0}".format(local_target),
+                Block([loop_body_prelude] + loop_body))
+        stmts = [prelude, loop]
 
         # in that case when can proceed to a reserve
         for comp in metadata.get(node, metadata.Comprehension):
@@ -599,9 +617,9 @@ class Cxx(Backend):
                         local_iter)))
 
         if break_handler:
-            stmts.append(Block(
-                [self.visit(n) for n in node.orelse]
-                + [Statement("{0}:".format(break_handler))]))
+            orelse = map(self.visit, node.orelse)
+            orelse_label = Statement("{0}:".format(break_handler))
+            stmts.append(Block(orelse + [orelse_label]))
 
         return Block(self.process_omp_attachements(node, stmts, 1))
 
@@ -626,11 +644,11 @@ class Cxx(Backend):
         body = [self.visit(m) for m in node.body]
         if not isinstance(node.type, ast.Tuple):
             return [ExceptHandler(
-                node.type.id if node.type else None,
+                node.type and node.type.attr,
                 Block(body),
                 name)]
         else:
-            elts = [p.id for p in node.type.elts]
+            elts = [p.attr for p in node.type.elts]
             return [ExceptHandler(o, Block(body), name) for o in elts]
 
     def visit_If(self, node):
@@ -658,9 +676,9 @@ class Cxx(Backend):
             return Statement("throw {0}".format(type or ""))
 
     def visit_Assert(self, node):
-        params = [node.msg and self.visit(node.msg), self.visit(node.test)]
-        return Statement("assert(({0}))".format(
-            ", ".join(p for p in params if p)))
+        params = [self.visit(node.test), node.msg and self.visit(node.msg)]
+        sparams = ", ".join(map(strip_exp, filter(None, params)))
+        return Statement("pythran_assert({0})".format(sparams))
 
     def visit_Import(self, node):
         return EmptyStatement()  # everything is already #included
@@ -669,7 +687,12 @@ class Cxx(Backend):
         assert False, "should be filtered out by the expand_import pass"
 
     def visit_Expr(self, node):
-        stmt = Statement(self.visit(node.value))
+        # turn docstring into comments
+        if type(node.value) is ast.Str:
+            stmt = Line("//" + node.value.s.replace('\n', '\n//'))
+        # other expressions are processed normally
+        else:
+            stmt = Statement(self.visit(node.value))
         return self.process_omp_attachements(node, stmt)
 
     def visit_Pass(self, node):
@@ -713,7 +736,7 @@ class Cxx(Backend):
 
     def visit_List(self, node):
         if not node.elts:  # empty list
-            return "list()"
+            return "__builtin__::list()"
         else:
             elts = [self.visit(n) for n in node.elts]
             return "{0}({{ {1} }})".format(
@@ -722,7 +745,7 @@ class Cxx(Backend):
 
     def visit_Set(self, node):
         if not node.elts:  # empty set
-            return "set()"
+            return "__builtin__::set()"
         else:
             elts = [self.visit(n) for n in node.elts]
             return "{0}({{ {1} }})".format(
@@ -731,7 +754,7 @@ class Cxx(Backend):
 
     def visit_Dict(self, node):
         if not node.keys:  # empty dict
-            return "dict()"
+            return "__builtin__::dict()"
         else:
             keys = [self.visit(n) for n in node.keys]
             values = [self.visit(n) for n in node.values]
@@ -741,18 +764,21 @@ class Cxx(Backend):
                         for k, v in zip(keys, values)))
 
     def visit_Tuple(self, node):
-        if not node.elts:  # empty tuple
-            return "std::tuple<>()"
-        else:
-            elts = [self.visit(n) for n in node.elts]
-            return "std::make_tuple({0})".format(", ".join(elts))
+        elts = map(self.visit, node.elts or ())
+        return "core::make_tuple({0})".format(", ".join(elts))
 
     def visit_Compare(self, node):
         left = self.visit(node.left)
         ops = [operator_to_lambda[type(n)] for n in node.ops]
         comparators = [self.visit(n) for n in node.comparators]
         all_compare = zip(ops, comparators)
-        return " and ".join(op(left, r) for op, r in all_compare)
+        op, right = all_compare[0]
+        output = [op(left, right)]
+        left = right
+        for op, right in all_compare[1:]:
+            output.append(op(left, right))
+            left = right
+        return " and ".join(output)
 
     def visit_Call(self, node):
         args = [self.visit(n) for n in node.args]
@@ -767,34 +793,35 @@ class Cxx(Backend):
                     repr(node.n.imag))
         elif type(node.n) == long:
             return 'pythran_long({0})'.format(node.n)
+        elif isnan(node.n):
+            return 'pythonic::nan'
         else:
             return repr(node.n) + type_to_suffix.get(type(node.n), "")
 
     def visit_Str(self, node):
-        return 'core::string("{0}")'.format(node.s.replace('\n', '\\n"\n"'))
+        quoted = node.s.replace('"', '\\"').replace('\n', '\\n"\n"')
+        return 'core::string("{0}")'.format(quoted)
 
     def visit_Attribute(self, node):
-        value, attr = (node.value, node.attr)
-        if (isinstance(value, ast.Name)
-                and value.id in modules
-                and attr in modules[value.id]):
-            if modules[value.id][attr]:
-                return "{0}::{1}".format(value.id, attr)
-            else:
-                return "{0}::proxy::{1}()".format(value.id, attr)
-        else:
-            raise PythranSyntaxError(
-                    "Attributes are only supported for namespaces",
-                    node)
+        def rec(w, n):
+            if isinstance(n, ast.Name):
+                return w[n.id], (n.id,)
+            elif isinstance(n, ast.Attribute):
+                r = rec(w, n.value)
+                return r[0][n.attr], r[1] + (n.attr,)
+        obj, path = rec(modules, node)
+        return ('::'.join(path) if obj.isliteral()
+                else ('::'.join(path[:-1]) + '::proxy::' + path[-1] + '()'))
 
     def visit_Subscript(self, node):
         value = self.visit(node.value)
         # attribute case
         if metadata.get(node, metadata.Attribute):
             return "getattr<{0}>({1})".format(node.slice.value.n, value)
-        # static index case
+        # positive static index case
         elif (isinstance(node.slice, ast.Index)
                 and isinstance(node.slice.value, ast.Num)
+                and (node.slice.value.n >= 0)
                 and any(isinstance(node.slice.value.n, t)
                     for t in (int, long))):
             return "std::get<{0}>({1})".format(node.slice.value.n, value)
@@ -804,18 +831,19 @@ class Cxx(Backend):
                     or node not in self.bounded_expressions)):
             slice = self.visit(node.slice)
             return "{1}({0})".format(slice, value)
+        # extended slice case
+        elif isinstance(node.slice, ast.ExtSlice):
+            slice = self.visit(node.slice)
+            return "{1}({0})".format(','.join(slice), value)
         # standard case
         else:
             slice = self.visit(node.slice)
             return "{1}[{0}]".format(slice, value)
 
     def visit_Name(self, node):
-        if node.id in self.local_declarations[-1]:
+        if node.id in self.local_declarations:
             return node.id
-        elif node.id in modules["__builtins__"]:
-            return "proxy::{0}()".format(node.id)
-        elif (node.id in self.global_declarations
-                or node.id in self.local_functions):
+        elif node.id in self.global_declarations:
             return "{0}()".format(node.id)
         elif node.id in builtin_constructors:
             return "pythonic::constructor<{0}>()".format(
@@ -824,6 +852,9 @@ class Cxx(Backend):
             return node.id
 
     # other
+    def visit_ExtSlice(self, node):
+        return map(self.visit, node.dims)
+
     def visit_Slice(self, node):
         lower = node.lower and self.visit(node.lower)
         upper = node.upper and self.visit(node.upper)
