@@ -5,7 +5,6 @@ This module provides a way to pass information between passes as metadata.
 '''
 
 from ast import AST  # so that metadata are walkable as regular ast nodes
-import ast
 
 
 class Metadata(AST):
@@ -28,9 +27,14 @@ class Attribute(AST):
     pass
 
 
+class Lazy(AST):
+    pass
+
+
 class Comprehension(AST):
-    def __init__(self, target):
-        self.target = target
+    def __init__(self, *args):  # no positional argument to be deep copyable
+        if args:
+            self.target = args[0]
 
 
 def add(node, data):
