@@ -1,25 +1,31 @@
 '''
 This modules contains code transformation to turn python AST into
     pythran AST
-    * NormalizeTuples removes implicite variable -> tuple conversion
+    * NormalizeTuples removes implicit variable -> tuple conversion
     * RemoveComprehension turns list comprehension into function calls
     * RemoveNestedFunctions turns nested function into top-level functions
     * RemoveLambdas turns lambda into regular functions
     * NormalizeReturn adds return statement where relevant
     * NormalizeMethodCalls turns built in method calls into function calls
+    * NormalizeCompare turns complex compare into function calls
     * NormalizeAttributes turns built in attributes into function calls
     * NormalizeIdentifiers prevents conflicts with c++ keywords
     * NormalizeException simplifies try blocks
     * UnshadowParameters prevents the shadow parameter phenomenon
     * ExpandImports replaces imports by their full paths
+    * ExpandBuiltins replaces builtins by their full paths
+    * FalsePolymorphism rename variable if possible to avoid false polymorphism
 '''
 
-from analysis import ImportedIds, Identifiers, YieldPoints
+from analysis import ImportedIds, Identifiers, YieldPoints, Globals, Locals
+from analysis import UsedDefChain, UseOMP
 from passmanager import Transformation
-from tables import methods, attributes, functions
-from tables import cxx_keywords, namespace
+from tables import methods, attributes, functions, modules
+from tables import builtin_constructors
+from tables import cxx_keywords, namespace, builtin_constants
 from operator import itemgetter
 from copy import copy
+import networkx as nx
 import metadata
 import ast
 
@@ -54,7 +60,6 @@ class NormalizeTuples(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(NormalizeTuples, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
     a = (1, 2.0)
     if 1:
         __tuple10 = a
@@ -93,9 +98,10 @@ class NormalizeTuples(Transformation):
         else:
             return node
 
-    def visit_AnyComp(self, node):
-        self.generic_visit(node.elt)
-        generators = [self.visit(n) for n in node.generators]
+    def visit_AnyComp(self, node, *fields):
+        for field in fields:
+            setattr(node, field, self.visit(getattr(node, field)))
+        generators = map(self.visit, node.generators)
         nnode = node
         for i, g in enumerate(generators):
             if isinstance(g, tuple):
@@ -107,18 +113,22 @@ class NormalizeTuples(Transformation):
                         nnode.generators[i].target,
                         metadata.LocalVariable())
                 nnode = _ConvertToTuple(gtarget, g[1]).visit(nnode)
-        node.elt = nnode.elt
+        for field in fields:
+            setattr(node, field, getattr(nnode, field))
         node.generators = nnode.generators
         return node
 
     def visit_ListComp(self, node):
-        return self.visit_AnyComp(node)
+        return self.visit_AnyComp(node, 'elt')
 
     def visit_SetComp(self, node):
-        return self.visit_AnyComp(node)
+        return self.visit_AnyComp(node, 'elt')
+
+    def visit_DictComp(self, node):
+        return self.visit_AnyComp(node, 'key', 'value')
 
     def visit_GeneratorExp(self, node):
-        return self.visit_AnyComp(node)
+        return self.visit_AnyComp(node, 'elt')
 
     def visit_Lambda(self, node):
         self.generic_visit(node)
@@ -174,22 +184,30 @@ class NormalizeTuples(Transformation):
             renamings = dict()
             self.traverse_tuples(target, (), renamings)
             if renamings:
-                elems = [x[0] for x in
-                        sorted(renamings.items(), key=itemgetter(1))]
                 self.counter += 1
-                newname = "{0}{1}".format(
+                gtarget = "{0}{1}".format(
                         NormalizeTuples.tuple_name,
-                        self.counter)
-                node.target = ast.Name(newname, node.target.ctx)
-                metadata.add(node.target, metadata.LocalVariable())
-                node.body.insert(0,
-                        ast.Assign([
-                            ast.Tuple(
-                                [ast.Name(x, ast.Store()) for x in elems],
-                                ast.Store())],
-                            ast.Name(newname, ast.Load())
-                            )
+                        self.counter
                         )
+                node.target = ast.Name(gtarget, node.target.ctx)
+                metadata.add(node.target, metadata.LocalVariable())
+                for rename, state in sorted(renamings.iteritems()):
+                    nnode = reduce(
+                            lambda x, y: ast.Subscript(
+                                x,
+                                ast.Index(ast.Num(y)),
+                                ast.Load()),
+                            state,
+                            ast.Name(gtarget, ast.Load()))
+                    if isinstance(rename, str):
+                        node.body.insert(0,
+                                ast.Assign(
+                                    [ast.Name(rename, ast.Store())],
+                                    nnode)
+                                )
+                    else:
+                        node.body.insert(0, ast.Assign([rename], nnode))
+
         self.generic_visit(node)
         return node
 
@@ -204,11 +222,9 @@ class RemoveComprehension(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(RemoveComprehension, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
     list_comprehension0()
-    <BLANKLINE>
     def list_comprehension0():
-        __target = list()
+        __target = __builtin__.list()
         for x in (1, 2, 3):
             __list__.append(__target, (x * x))
         return __target
@@ -250,10 +266,16 @@ class RemoveComprehension(Transformation):
         metadata.add(body, metadata.Comprehension(starget))
         init = ast.Assign(
                 [ast.Name(starget, ast.Store())],
-                ast.Call(ast.Name(comp_type, ast.Load()), [], [], None, None)
+                ast.Call(
+                    ast.Attribute(
+                        ast.Name('__builtin__', ast.Load()),
+                        comp_type,
+                        ast.Load()
+                        ),
+                    [], [], None, None)
                 )
         result = ast.Return(ast.Name(starget, ast.Load()))
-        sargs = sorted(ast.Name(arg, ast.Load()) for arg in args)
+        sargs = sorted(ast.Name(arg, ast.Param()) for arg in args)
         fd = ast.FunctionDef(name,
                 ast.arguments(sargs, None, None, []),
                 [init, body, result],
@@ -294,7 +316,7 @@ class RemoveComprehension(Transformation):
                 ast.Expr(ast.Yield(node.elt))
                 )
 
-        sargs = sorted(ast.Name(arg, ast.Load()) for arg in args)
+        sargs = sorted(ast.Name(arg, ast.Param()) for arg in args)
         fd = ast.FunctionDef(name,
                 ast.arguments(sargs, None, None, []),
                 [body],
@@ -349,7 +371,11 @@ class _NestedFunctionRemover(Transformation):
         new_node = ast.Assign(
                 [ast.Name(former_name, ast.Store())],
                 ast.Call(
-                    ast.Name("bind{0}".format(former_nbargs), ast.Load()),
+                    ast.Attribute(
+                        ast.Name('__builtin__', ast.Load()),
+                        "bind{0}".format(former_nbargs),
+                        ast.Load()
+                        ),
                     [proxy_call] + binded_args,
                     [],
                     None,
@@ -372,12 +398,9 @@ class RemoveNestedFunctions(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(RemoveNestedFunctions, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
-    <BLANKLINE>
     def foo(x):
-        bar = bind1(pythran_bar, x)
+        bar = __builtin__.bind1(pythran_bar, x)
         bar(12)
-    <BLANKLINE>
     def pythran_bar(x, y):
         return (x + y)
     '''
@@ -403,11 +426,14 @@ class _LambdaRemover(Transformation):
         self.lambda_functions = list()
 
     def visit_Lambda(self, node):
-        node.body = self.visit(node.body)
+        self.generic_visit(node)
         forged_name = "{0}_lambda{1}".format(
                 self.prefix,
                 len(self.lambda_functions))
+
         ii = self.passmanager.gather(ImportedIds, node, self.ctx)
+        ii.difference_update(self.lambda_functions)  # remove current lambdas
+
         binded_args = [ast.Name(iin, ast.Load()) for iin in sorted(ii)]
         former_nbargs = len(node.args.args)
         node.args.args = ([ast.Name(iin, ast.Param()) for iin in sorted(ii)]
@@ -420,7 +446,11 @@ class _LambdaRemover(Transformation):
         self.lambda_functions.append(forged_fdef)
         proxy_call = ast.Name(forged_name, ast.Load())
         return ast.Call(
-                ast.Name("bind{0}".format(former_nbargs), ast.Load()),
+                ast.Attribute(
+                    ast.Name('__builtin__', ast.Load()),
+                    "bind{0}".format(former_nbargs),
+                    ast.Load()
+                    ),
                 [proxy_call] + binded_args,
                 [],
                 None,
@@ -436,18 +466,15 @@ class RemoveLambdas(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(RemoveLambdas, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
-    <BLANKLINE>
     def foo(y):
-        bind1(foo_lambda0, y)
-    <BLANKLINE>
+        __builtin__.bind1(foo_lambda0, y)
     def foo_lambda0(y, x):
         return (y + x)
     '''
 
     def visit_Module(self, node):
         self.lambda_functions = list()
-        [self.visit(n) for n in node.body]
+        self.generic_visit(node)
         node.body.extend(self.lambda_functions)
         return node
 
@@ -469,8 +496,6 @@ class NormalizeReturn(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(NormalizeReturn, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
-    <BLANKLINE>
     def foo(y):
         print y
         return None
@@ -508,18 +533,73 @@ class NormalizeMethodCalls(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(NormalizeMethodCalls, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
     __list__.append(l, 12)
     '''
 
-    def visit_Call(self, node):
+    def __init__(self):
+        Transformation.__init__(self, Globals)
+        self.imports = set()
+
+    def visit_FunctionDef(self, node):
+        self.imports = self.globals.copy()
+        [self.imports.discard(arg.id) for arg in node.args.args]
         self.generic_visit(node)
-        if isinstance(node.func, ast.Attribute) and node.func.attr in methods:
-            node.args.insert(0,  node.func.value)
-            node.func = ast.Attribute(
-                    ast.Name(methods[node.func.attr][0], ast.Load()),
-                    node.func.attr,
-                    ast.Load())
+        return node
+
+    def visit_Import(self, node):
+        for alias in node.names:
+            self.imports.add(alias.asname or alias.name)
+        return node
+
+    def visit_Assign(self, node):
+        n = self.generic_visit(node)
+        for t in node.targets:
+            if isinstance(t, ast.Name):
+                self.imports.discard(t.id)
+        return n
+
+    def visit_For(self, node):
+        node.iter = self.visit(node.iter)
+        if isinstance(node.target, ast.Name):
+            self.imports.discard(node.target.id)
+        if node.body:
+            node.body = [self.visit(n) for n in node.body]
+        if node.orelse:
+            node.orelse = [self.visit(n) for n in node.orelse]
+        return node
+
+    def visit_Call(self, node):
+        node = self.generic_visit(node)
+        if isinstance(node.func, ast.Attribute):
+            lhs = node.func.value
+            if node.func.attr in methods:
+                isname = isinstance(lhs, ast.Name)
+                ispath = isname or isinstance(lhs, ast.Attribute)
+                if not ispath or (isname and lhs.id not in self.imports):
+                    node.args.insert(0, node.func.value)
+                    node.func = ast.Attribute(
+                            ast.Name(methods[node.func.attr][0], ast.Load()),
+                            node.func.attr,
+                            ast.Load())
+            if node.func.attr in methods or node.func.attr in functions:
+                def renamer(v):
+                    name = '__{0}__'.format(v)
+                    if name in modules:
+                        return name
+                    else:
+                        name += '_'
+                        if name in modules:
+                            return name
+                    return v
+
+                def rec(n):
+                    if isinstance(n, ast.Attribute):
+                        n.attr = renamer(n.attr)
+                        rec(n.value)
+                    elif isinstance(n, ast.Name):
+                        n.id = renamer(n.id)
+                rec(node.func.value)
+
         return node
 
 
@@ -533,7 +613,6 @@ class NormalizeAttributes(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(NormalizeAttributes, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
     a[0]
     '''
 
@@ -545,14 +624,6 @@ class NormalizeAttributes(Transformation):
                     node.ctx)
             metadata.add(out, metadata.Attribute())
             return out
-        elif node.attr in functions:
-            if len(functions[node.attr]) > 1:
-                return node
-            else:
-                [(module, _)] = functions[node.attr]
-                assert isinstance(node.value, ast.Name)
-                node.value.id = module
-                return node
         else:
             return node
 
@@ -568,8 +639,6 @@ class NormalizeIdentifiers(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> d = pm.apply(NormalizeIdentifiers, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
-    <BLANKLINE>
     def namespace_(union_):
         pass
     '''
@@ -586,8 +655,8 @@ class NormalizeIdentifiers(Transformation):
             self.renamings[name] = new_name
         return self.renamings[name]
 
-    def run_visit(self, node):
-        self.visit(node)
+    def run(self, node, ctx):
+        super(NormalizeIdentifiers, self).run(node, ctx)
         return self.renamings
 
     def visit_Name(self, node):
@@ -603,9 +672,17 @@ class NormalizeIdentifiers(Transformation):
         return node
 
     def visit_alias(self, node):
+        if node.name in cxx_keywords:
+            node.name = self.rename(node.name)
         if node.asname:
             if node.asname in cxx_keywords:
-                node.asname = self.rename(node.name)
+                node.asname = self.rename(node.asname)
+        return node
+
+    def visit_ImportFrom(self, node):
+        self.generic_visit(node)
+        if node.module and node.module in cxx_keywords:
+            node.module = self.rename(node.module)
         return node
 
     def visit_Attribute(self, node):
@@ -626,7 +703,6 @@ class NormalizeException(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(NormalizeException, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
     try:
         print 't'
         try:
@@ -667,8 +743,6 @@ class UnshadowParameters(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(UnshadowParameters, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
-    <BLANKLINE>
     def foo(a):
         a_ = a
         a_ = None
@@ -693,22 +767,26 @@ class UnshadowParameters(Transformation):
                     )
         return node
 
+    def update(self, node):
+        if isinstance(node, ast.Name) and node.id in self.argsid:
+            if node.id not in self.renaming:
+                new_name = node.id
+                while new_name in self.identifiers:
+                    new_name = new_name + "_"
+                self.renaming[node.id] = new_name
+
     def visit_Assign(self, node):
-        for t in node.targets:
-            if isinstance(t, ast.Name) and t.id in self.argsid:
-                if t.id not in self.renaming:
-                    new_name = t.id
-                    while new_name in self.identifiers:
-                        new_name = new_name + "_"
-                    self.renaming[t.id] = new_name
-        [self.visit(t) for t in node.targets]
+        map(self.update, node.targets)
         try:
-            self.visit(node.metadata)
+            self.generic_visit(node)
         except AttributeError:
             pass
-        self.visit(node.value)
         return node
 
+    def visit_AugAssign(self, node):
+        self.update(node.target)
+        return self.generic_visit(node)
+
     def visit_Name(self, node):
         if node.id in self.renaming:
             node.id = self.renaming[node.id]
@@ -724,7 +802,6 @@ class ExpandImports(Transformation):
     >>> pm = passmanager.PassManager("test")
     >>> node = pm.apply(ExpandImports, node)
     >>> print pm.dump(backend.Python, node)
-    <BLANKLINE>
     import math as pythonic::math
     math.cos(2)
     '''
@@ -766,11 +843,10 @@ class ExpandImports(Transformation):
         return node
 
     def visit_Assign(self, node):
-        self.visit(node.value)
-        [self.visit(n) for n in node.targets]
+        new_node = self.generic_visit(node)
         [self.symbols.pop(t.id, None)
-                for t in node.targets if isinstance(t, ast.Name)]
-        return node
+                for t in new_node.targets if isinstance(t, ast.Name)]
+        return new_node
 
     def visit_Name(self, node):
         if node.id in self.symbols:
@@ -783,3 +859,145 @@ class ExpandImports(Transformation):
             ast.copy_location(new_node, node)
             return new_node
         return node
+
+
+class ExpandBuiltins(Transformation):
+    '''
+    Expands all builtins into full paths.
+    >>> import ast, passmanager, backend
+    >>> node = ast.parse("def foo(): return list()")
+    >>> pm = passmanager.PassManager("test")
+    >>> node = pm.apply(ExpandBuiltins, node)
+    >>> print pm.dump(backend.Python, node)
+    def foo():
+        return __builtin__.list()
+    '''
+
+    def __init__(self):
+        Transformation.__init__(self, Locals, Globals)
+
+    def visit_Name(self, node):
+        s = node.id
+        if (isinstance(node.ctx, ast.Load)
+                and s not in builtin_constants
+                and s not in builtin_constructors
+                and s not in self.locals[node]
+                and s not in self.globals
+                and s in modules['__builtin__']):
+            return ast.Attribute(
+                    ast.Name('__builtin__', ast.Load()),
+                    s,
+                    node.ctx)
+        else:
+            return node
+
+
+class FalsePolymorphism(Transformation):
+    """
+    Rename variable when possible to avoid false polymorphism.
+    >>> import ast, passmanager, backend
+    >>> node = ast.parse("def foo(): a = 12; a = 'babar'")
+    >>> pm = passmanager.PassManager("test")
+    >>> node = pm.apply(FalsePolymorphism, node)
+    >>> print pm.dump(backend.Python, node)
+    def foo():
+        a = 12
+        a_ = 'babar'
+    """
+    def __init__(self):
+        super(FalsePolymorphism, self).__init__(UsedDefChain, UseOMP)
+
+    def visit_FunctionDef(self, node):
+        #function using openmp are ignored
+        if not self.use_omp:
+            self.identifiers = self.passmanager.gather(Identifiers, node,
+                    self.ctx)
+            for name, udgraph in self.used_def_chain.iteritems():
+                group_variable = list()
+                while udgraph:
+                    e = udgraph.nodes_iter().next()
+                    to_change = set()
+                    to_analyse_pred = set([e])
+                    to_analyse_succ = set()
+                    while to_analyse_pred or to_analyse_succ:
+                        if to_analyse_pred:
+                            n = to_analyse_pred.pop()
+                            to_change.add(n)
+                            to_analyse_succ.update(udgraph.successors(n))
+                            to_analyse_succ -= to_change
+                        else:
+                            n = to_analyse_succ.pop()
+                            if (udgraph.node[n]['action'] == 'U' or
+                                    udgraph.node[n]['action'] == 'UD'):
+                                to_change.add(n)
+                                to_analyse_succ.update(udgraph.successors(n))
+                                to_analyse_succ -= to_change
+                        if (udgraph.node[n]['action'] == 'U' or
+                                udgraph.node[n]['action'] == 'UD'):
+                            to_analyse_pred.update(udgraph.predecessors(n))
+                            to_analyse_pred -= to_change
+                    nodes_to_change = [udgraph.node[k]['name']
+                                            for k in to_change]
+                    group_variable.append(nodes_to_change)
+                    udgraph.remove_nodes_from(to_change)
+                if len(group_variable) > 1:
+                    self.identifiers.remove(name)
+                    for group in group_variable:
+                        while name in self.identifiers:
+                            name += "_"
+                        for var in group:
+                            var.id = name
+                        self.identifiers.add(name)
+        return node
+
+
+class NormalizeCompare(Transformation):
+    '''
+    Turns multiple compare into a function with proper temporaries.
+
+    >>> import ast, passmanager, backend
+    >>> node = ast.parse("def foo(a): return 0 < a + 1 < 3")
+    >>> pm = passmanager.PassManager("test")
+    >>> node = pm.apply(NormalizeCompare, node)
+    >>> print pm.dump(backend.Python, node)
+    def foo(a):
+        return foo_compare0(0, (a + 1), 3)
+    def foo_compare0($0, $1, $2):
+        return ($0 < $1 < $2)
+    '''
+
+    def visit_Module(self, node):
+        self.compare_functions = list()
+        self.generic_visit(node)
+        node.body.extend(self.compare_functions)
+        return node
+
+    def visit_FunctionDef(self, node):
+        self.prefix = node.name
+        self.generic_visit(node)
+        return node
+
+    def visit_Compare(self, node):
+        node = self.generic_visit(node)
+        if len(node.ops) > 1:
+            forged_name = "{0}_compare{1}".format(
+                    self.prefix,
+                    len(self.compare_functions)
+                    )
+            binded_args = [node.left] + node.comparators
+            args = ast.arguments([ast.Name('${}'.format(i), ast.Param())
+                for i in range(1 + len(node.ops))],
+                None, None, [])
+            node.left = ast.Name('$0', ast.Load())
+            node.comparators = [ast.Name('${}'.format(i), ast.Load())
+                    for i in range(1, 1 + len(node.ops))]
+            forged_fdef = ast.FunctionDef(
+                    forged_name,
+                    args,
+                    [ast.Return(node)],
+                    [])
+            self.compare_functions.append(forged_fdef)
+            return ast.Call(ast.Name(forged_name, ast.Load()),
+                    binded_args, [], None, None)
+        else:
+            return node
