@@ -10,6 +10,7 @@ namespace pythonic {
         class function {
             char mem[sizeof(T)];
             public:
+            typedef void callable;
             T* ptr; //SG: I fail to understand why gcc requires this to be public
 
             function() : ptr(nullptr) {}
@@ -32,6 +33,9 @@ namespace pythonic {
 
             variant() : t(nullptr) {}
             variant(Type const& t) : t(new (mem) Type(t)) {}
+            variant(variant<Type> const& t) : t(t.t?new (mem) Type(*t.t):t.t) {}
+            template<class OtherType>
+            variant(variant<OtherType> const& t) : t(nullptr) {}
 
             template <class... Args>
                 auto operator()(Args&&... args) -> decltype( std::declval<Type>()(args...)){
@@ -42,6 +46,7 @@ namespace pythonic {
 
     template<class Type, class... Types>
         struct variant<Type, Types...> {
+            typedef void callable;
             char mem[sizeof(Type)];
             Type* t;
             variant<Types...> ot;
@@ -51,6 +56,18 @@ namespace pythonic {
             template<class T>
                 variant(T const& t) : t(nullptr), ot(t) {}
 
+            // the below constructor handle case when Types and OtherTypes are not exactly the same
+            // or are in a different order
+            template<class OtherType, class... OtherTypes>
+                variant(variant<OtherType, OtherTypes...> const& t) :
+                    t(nullptr),
+                    ot(t.ot) {}
+
+            template<class... OtherTypes>
+                variant(variant<Type, OtherTypes...> const& t) :
+                    t(t.t?new (mem) Type(*t.t):t.t),
+                    ot(t.ot) {}
+
             template <class... Args>
                 auto operator()(Args&&... args) -> decltype( std::declval<Type>()(args...)){
                     if(t) return (*t)(args...);
