@@ -61,6 +61,7 @@ namespace  pythonic {
                 // assignment
                 list_view& operator=(list<T> const & );
                 list_view& operator=(list_view<T> const & );
+                list<T> operator+(list<T> const & );
                 list<T> operator+(list_view<T> const & );
 
                 // iterators
@@ -79,13 +80,8 @@ namespace  pythonic {
                 // comparison
                 template <class K>
                     bool operator==(core::list<K> const & other) const {
-                        auto self_iter=begin();
-                        for(auto const & other_iter : other) {
-                            if(self_iter == end() ) return false;
-                            else if(*self_iter != other_iter) return false;
-                            ++self_iter;
-                        }
-                        return self_iter == end();
+                        if(size()!=other.size()) return false;
+                        return std::equal(begin(),end(),other.begin());
                     }
 
 
@@ -132,15 +128,18 @@ namespace  pythonic {
                         data->reserve(DEFAULT_LIST_CAPACITY);
                         std::copy(start, stop, std::back_inserter(*data));
                     }
-                list(empty_list const &) : data() {}
                 list(size_type sz) :data(sz) {}
-                list(std::initializer_list<value_type> l) : data(std::move(l)) {}
+                list(std::initializer_list<T> l) : data(std::move(l)) {}
                 list(list<T> && other) : data(std::move(other.data)) {}
                 list(list<T> const & other) : data(other.data) {}
                 template<class F>
                     list(list<F> const & other) : data(other.size()) {
                         std::copy(other.begin(), other.end(), begin());
                     }
+                template<class... Types>
+                    list(std::tuple<Types...> const& t) : data(sizeof...(Types)) {
+                        tuple_dump(t, *this, int_<sizeof...(Types)-1>());
+                    }
                 list(list_view<T> const & other) : data( other.begin(), other.end()) {}
 
                 list<T>& operator=(list<T> && other) {
@@ -178,6 +177,32 @@ namespace  pythonic {
                     return new_list;
                 }
 
+                // io
+
+                /* list */
+
+                friend std::ostream& operator<<(std::ostream& os, core::list<T> const & v) {
+                    os << '[';
+                    auto iter = v.begin();
+                    if(iter != v.end()) {
+                        while(iter+1 != v.end())
+                            os << *iter++ << ", ";
+                        os << *iter;
+                    }
+                    return os << ']';
+                }
+
+                // comparison
+                template <class K>
+                    bool operator==(core::list<K> const & other) const {
+                        if(size()!=other.size()) return false;
+                        return std::equal(begin(),end(),other.begin());
+                    }
+                template <class K>
+                    bool operator!=(core::list<K> const & other) const {
+                        return !operator==(other);
+                    }
+
                 // iterators
                 iterator begin() { return data->begin(); }
                 const_iterator begin() const { return data->begin(); }
@@ -208,6 +233,9 @@ namespace  pythonic {
                 const_reference operator[]( long n ) const {
                     return (*data)[(n>=0)?n : (data->size() + n)];
                 }
+                const_reference at( long n ) const {
+                    return (*data)[n];
+                }
 
                 list<T> operator[]( slice const &s ) const {
                     list<T> out(0);
@@ -264,13 +292,14 @@ namespace  pythonic {
                 }
 
 
+
                 //Misc
                 //TODO: have to raise a valueError
-                long index(T const& x) {
-                    return std::find(begin(),end(),x)-begin();
-                }	
+                long index(T const& x) const { return std::find(begin(),end(),x)-begin(); }	
+
 
                 // list interface
+                explicit
                 operator bool() const { return not data->empty(); }
 
                 template <class F>
@@ -315,12 +344,14 @@ namespace  pythonic {
 
         /* empty list implementation */
         struct empty_list {
-            template<class T> // just for type inference, should never been instantiated
-                list<T> operator+(list<T> const & s) { return s; }
-            template<class T> // just for type inference, should never been instantiated
-                list_view<T> operator+(list_view<T> const & s) { return s; }
-            empty_list operator+(empty_list const &) { return empty_list(); }
-            operator bool() { return false; }
+            template<class T>
+                list<T> operator+(list<T> const & s) const { return s; }
+            template<class T>
+                list_view<T> operator+(list_view<T> const & s) const { return s; }
+            empty_list operator+(empty_list const &) const { return empty_list(); }
+            operator bool() const { return false; }
+            template<class T>
+                operator list<T>() const { return list<T>(0); }
         };
 
         /* list_view implementation */
@@ -369,6 +400,12 @@ namespace  pythonic {
                 }
                 return *this;
             }
+        template<class T>
+            list<T> list_view<T>::operator+(list<T> const & s) {
+                list<T> out(size() + s.size());
+                std::copy(s.begin(), s.end(), std::copy(begin(), end(), out.begin()));
+                return out;
+            }
         template<class T>
             list<T> list_view<T>::operator+(list_view<T> const & s) {
                 list<T> out(size() + s.size());
@@ -383,4 +420,17 @@ namespace  pythonic {
     }
 
 }
+
+/* overload std::get */
+namespace std {
+    template <size_t I, class T>
+        typename pythonic::core::list<T>::reference get( pythonic::core::list<T>& t) { return t[I]; }
+    template <size_t I, class T>
+        typename pythonic::core::list<T>::const_reference get( pythonic::core::list<T> const & t) { return t[I]; }
+
+    template <size_t I, class T>
+        struct tuple_element<I, pythonic::core::list<T> > {
+            typedef typename pythonic::core::list<T>::value_type type;
+        };
+}
 #endif
