@@ -5,19 +5,24 @@ This module performs the return type inference, according to symbolic types,
 '''
 
 import ast
+from numpy import ndarray
 import networkx as nx
+
 from tables import pytype_to_ctype_table, operator_to_lambda
 from tables import modules, builtin_constants, builtin_constructors
 from tables import methods, functions
 from analysis import GlobalDeclarations, YieldPoints, LocalDeclarations
 from analysis import OrderedGlobalDeclarations, ModuleAnalysis, StrictAliases
+from analysis import LazynessAnalysis
 from passes import Transformation
 from syntax import PythranSyntaxError
 from cxxtypes import *
 from intrinsic import UserFunction, MethodIntr
-import types
 import itertools
+import operator
 import metadata
+import intrinsic
+from config import cfg
 
 
 # networkx backward compatibility
@@ -32,13 +37,56 @@ if not "has_path" in nx.__dict__:
 
 
 ##
-def add_if_not_in(l0, l1):
-    s0 = set(l0)
-    l = [x for x in l0]
-    return l + [x for x in l1 if x not in s0]
+def pytype_to_ctype(t):
+    '''python -> c++ type binding'''
+    if isinstance(t, list):
+        return 'core::list<{0}>'.format(pytype_to_ctype(t[0]))
+    if isinstance(t, set):
+        return 'core::set<{0}>'.format(pytype_to_ctype(list(t)[0]))
+    elif isinstance(t, dict):
+        tkey, tvalue = t.items()[0]
+        return 'core::dict<{0},{1}>'.format(pytype_to_ctype(tkey),
+                                            pytype_to_ctype(tvalue))
+    elif isinstance(t, tuple):
+        return 'std::tuple<{0}>'.format(", ".join(pytype_to_ctype(_)
+                                        for _ in t))
+    elif isinstance(t, ndarray):
+        return 'core::ndarray<{0},{1}>'.format(pytype_to_ctype(t.flat[0]),
+                                               t.ndim)
+    elif t in pytype_to_ctype_table:
+        return pytype_to_ctype_table[t]
+    else:
+        raise NotImplementedError("{0}:{1}".format(type(t), t))
+
+
+def extract_constructed_types(t):
+    if isinstance(t, list) or isinstance(t, ndarray):
+        return [pytype_to_ctype(t)] + extract_constructed_types(t[0])
+    elif isinstance(t, set):
+        return [pytype_to_ctype(t)] + extract_constructed_types(list(t)[0])
+    elif isinstance(t, dict):
+        tkey, tvalue = t.items()[0]
+        return ([pytype_to_ctype(t)]
+                + extract_constructed_types(tkey)
+                + extract_constructed_types(tvalue))
+    elif isinstance(t, tuple):
+        return ([pytype_to_ctype(t)]
+                + sum(map(extract_constructed_types, t), []))
+    elif t == long:
+        return ["pythran_long_def"]
+    elif t == str:
+        return ["core::string"]
+    else:
+        return []
 
 
 class TypeDependencies(ModuleAnalysis):
+    '''
+    Gathers the callees of each function required for type inference
+
+    This analyse produces a directed graph with functions as nodes and edges
+    between nodes when a function might call another.
+    '''
 
     NoDeps = "None"
 
@@ -47,11 +95,21 @@ class TypeDependencies(ModuleAnalysis):
         self.current_function = None
         ModuleAnalysis.__init__(self, GlobalDeclarations)
 
-    def run_visit(self, node):
+    def prepare(self, node, ctx):
+        super(TypeDependencies, self).prepare(node, ctx)
         for k, v in self.global_declarations.iteritems():
             self.result.add_node(v)
         self.result.add_node(TypeDependencies.NoDeps)
-        return ModuleAnalysis.run_visit(self, node)
+
+    def visit_any_conditionnal(self, node):
+        '''
+        Set and restore the in_cond variable whenever a node
+        the children of which may not be executed is visited
+        '''
+        in_cond = self.in_cond
+        self.in_cond = True
+        self.generic_visit(node)
+        self.in_cond = in_cond
 
     def visit_FunctionDef(self, node):
         assert self.current_function is None
@@ -59,11 +117,19 @@ class TypeDependencies(ModuleAnalysis):
         self.current_function = node
         self.result.add_node(node)
         self.naming = dict()
-        [self.visit(n) for n in node.args.defaults]
-        [self.visit(n) for n in node.body]
+        self.in_cond = False  # True when we are in a if, while or for
+        self.generic_visit(node)
         self.current_function = None
 
     def visit_Return(self, node):
+        '''
+        Gather all the function call that led to the creation of the
+        returned expression and add an edge to each of this function.
+
+        When visiting an expression, one returns a list of frozensets.  Each
+        element of the list is linked to a possible path, each element of a
+        frozenset is linked to a dependency.
+        '''
         if node.value:
             v = self.visit(node.value)
             for dep_set in v:
@@ -74,32 +140,40 @@ class TypeDependencies(ModuleAnalysis):
                     self.result.add_edge(TypeDependencies.NoDeps,
                                             self.current_function)
 
-    def visit_Yield(self, node):
-        self.visit_Return(node)
+    visit_Yield = visit_Return
+
+    def update_naming(self, name, value):
+        '''
+        Update or renew the name <-> dependencies binding
+        depending on the in_cond state
+        '''
+        if self.in_cond:
+            self.naming.setdefault(name, []).extend(value)
+        else:
+            self.naming[name] = value
 
     def visit_Assign(self, node):
         v = self.visit(node.value)
-        self.naming.update({t.id: v  # need to handle subscript too ...
-            for t in node.targets if isinstance(t, ast.Name)})
+        for t in node.targets:
+            if isinstance(t, ast.Name):
+                self.update_naming(t.id, v)
 
     def visit_AugAssign(self, node):
         v = self.visit(node.value)
-        if isinstance(node.target, ast.Name):
-            self.naming.update({node.target.id: v})
+        t = node.target
+        if isinstance(t, ast.Name):
+            self.update_naming(t.id, v)
 
     def visit_For(self, node):
         self.naming.update({node.target.id: self.visit(node.iter)})
-        if node.body:
-            [self.visit(n) for n in node.body]
-        if node.orelse:
-            [self.visit(n) for n in node.orelse]
+        self.visit_any_conditionnal(node)
 
     def visit_BoolOp(self, node):
         return sum((self.visit(value) for value in node.values), [])
 
     def visit_BinOp(self, node):
-        return [l.union(r) for l in self.visit(node.left)
-                for r in self.visit(node.right)]
+        args = map(self.visit, (node.left, node.right))
+        return list({frozenset.union(*x) for x in itertools.product(*args)})
 
     def visit_UnaryOp(self, node):
         return self.visit(node.operand)
@@ -114,10 +188,10 @@ class TypeDependencies(ModuleAnalysis):
         return [frozenset()]
 
     def visit_Call(self, node):
+        args = map(self.visit, node.args)
         func = self.visit(node.func)
-        for arg in node.args:
-            func = [f.union(v) for f in func for v in self.visit(arg)]
-        return func
+        params = args + [func or []]
+        return list({frozenset.union(*p) for p in itertools.product(*params)})
 
     def visit_Num(self, node):
         return [frozenset()]
@@ -140,24 +214,21 @@ class TypeDependencies(ModuleAnalysis):
             return [frozenset()]
 
     def visit_List(self, node):
-        return reduce(add_if_not_in,
-                map(self.visit, node.elts),
-                [frozenset()])
+        if node.elts:
+            return list(set(sum(map(self.visit, node.elts), [])))
+        else:
+            return [frozenset()]
 
-    def visit_Set(self, node):
-        return reduce(add_if_not_in,
-                map(self.visit, node.elts),
-                [frozenset()])
+    visit_Set = visit_List
 
     def visit_Dict(self, node):
-        return reduce(add_if_not_in,
-                map(self.visit, node.keys) + map(self.visit, node.values),
-                [frozenset()])
+        if node.keys:
+            items = node.keys + node.values
+            return list(set(sum(map(self.visit, items), [])))
+        else:
+            return [frozenset()]
 
-    def visit_Tuple(self, node):
-        return reduce(add_if_not_in,
-                map(self.visit, node.elts),
-                [frozenset()])
+    visit_Tuple = visit_List
 
     def visit_Slice(self, node):
         return [frozenset()]
@@ -165,13 +236,20 @@ class TypeDependencies(ModuleAnalysis):
     def visit_Index(self, node):
         return [frozenset()]
 
+    visit_If = visit_any_conditionnal
+    visit_While = visit_any_conditionnal
+
 
 class Reorder(Transformation):
+    '''
+    Reorder top-level functions to prevent circular type dependencies
+    '''
     def __init__(self):
         Transformation.__init__(self, TypeDependencies,
                 OrderedGlobalDeclarations)
 
-    def run_visit(self, node):
+    def prepare(self, node, ctx):
+        super(Reorder, self).prepare(node, ctx)
         none_successors = self.type_dependencies.successors(
                 TypeDependencies.NoDeps)
         candidates = sorted(none_successors)
@@ -192,7 +270,6 @@ class Reorder(Transformation):
                 else:
                     pass
             candidates = new_candidates
-        Transformation.run_visit(self, node)
 
     def visit_Module(self, node):
         newbody = list()
@@ -203,10 +280,12 @@ class Reorder(Transformation):
             else:
                 newbody.append(stmt)
             try:
-                newdef = [f for f in nx.topological_sort(
+                newdef = (f for f in nx.topological_sort(
                     self.type_dependencies,
                     self.ordered_global_declarations
-                    ) if isinstance(f, ast.FunctionDef)]
+                    ))
+                newdef = [f for f in newdef if isinstance(f, ast.FunctionDef)]
+
             except nx.exception.NetworkXUnfeasible:
                 raise PythranSyntaxError("Infinite function recursion",
                                         stmt)
@@ -219,36 +298,32 @@ class UnboundableRValue(Exception):
     pass
 
 
-def copy_func(f, name=None):
-    return types.FunctionType(f.func_code,
-                            f.func_globals,
-                            name or f.func_name,
-                            f.func_defaults,
-                            f.func_closure)
-
-
 class Types(ModuleAnalysis):
-    '''Infer symbolic type for all AST node.'''
+    '''
+    Infer symbolic type for all AST node
+    '''
 
     def __init__(self):
         self.result = dict()
         self.result["bool"] = NamedType("bool")
-        self.current = list()
         self.current_global_declarations = dict()
-        ModuleAnalysis.__init__(self, StrictAliases, LocalDeclarations)
+        self.max_recompute = 1  # max number of use to be lazy
+        ModuleAnalysis.__init__(self, StrictAliases, LazynessAnalysis)
+
+    def prepare(self, node, ctx):
+        self.passmanager.apply(Reorder, node, ctx)
+        for mname, module in modules.iteritems():
+            for fname, function in module.iteritems():
+                tname = 'pythonic::{0}::proxy::{1}'.format(mname, fname)
+                self.result[function] = NamedType(tname)
+        super(Types, self).prepare(node, ctx)
 
     def run(self, node, ctx):
-        self.passmanager.apply(Reorder, node, ctx)
-        return ModuleAnalysis.run(self, node, ctx)
-
-    def run_visit(self, node):
-        ModuleAnalysis.run_visit(self, node)
+        super(Types, self).run(node, ctx)
         final_types = {k: self.result[k]
                 if k in self.result
                 else v
                 for k, v in self.result.iteritems()}
-        for k in self.local_declarations:
-            self.result[k] = Assignable(self.result[k])
         for head in self.current_global_declarations.itervalues():
             if head not in final_types:
                 final_types[head] = "void"
@@ -256,7 +331,11 @@ class Types(ModuleAnalysis):
 
     def register(self, ptype):
         """register ptype as a local typedef"""
-        self.typedefs.append(ptype)
+        # Too many of them leads to memory burst
+        if len(self.typedefs) < cfg.getint('typing', 'max_combiner'):
+            self.typedefs.append(ptype)
+            return True
+        return False
 
     def node_to_id(self, n, depth=0):
         if isinstance(n, ast.Name):
@@ -272,18 +351,21 @@ class Types(ModuleAnalysis):
             for alias in self.strict_aliases[func].aliases:
                 # handle backward type dependencies from method calls
                 signature = None
-                if isinstance(alias, ast.Attribute):
-                    _, signature = methods.get(
-                            func.attr,
-                            functions.get(func.attr, [(None, None)])[0]
-                            )
-                elif isinstance(alias, ast.Name):
-                    _, signature = functions.get(func.attr, [(None, None)])[0]
+                if isinstance(alias, MethodIntr):
+                    signature = alias
+                #if isinstance(alias, ast.Attribute):
+                #    _, signature = methods.get(
+                #            func.attr,
+                #            functions.get(func.attr, [(None, None)])[0]
+                #            )
+                #elif isinstance(alias, ast.Name):
+                #    _, signature = functions.get(func.attr, [(None, None)])[0]
                 if signature:
                     return_alias = (signature.return_alias
                             and signature.return_alias(n))
                     if return_alias:  # else new location -> unboundable
-                        return self.node_to_id(return_alias, depth)
+                        assert len(return_alias), 'Too many return aliases'
+                        return self.node_to_id(list(return_alias)[0], depth)
             raise UnboundableRValue()
 
         else:
@@ -301,8 +383,15 @@ class Types(ModuleAnalysis):
                 return False
 
     def combine(self, node, othernode, op=None, unary_op=None, register=False):
-        self.combine_(node, othernode, op or (lambda x, y: x + y),
+        if register and node in self.strict_aliases:
+            self.combine_(node, othernode, op or operator.add,
+                    unary_op or (lambda x: x), register)
+            for a in self.strict_aliases[node].aliases:
+                self.combine_(a, othernode, op or operator.add,
                         unary_op or (lambda x: x), register)
+        else:
+            self.combine_(node, othernode, op or operator.add,
+                    unary_op or (lambda x: x), register)
 
     def combine_(self, node, othernode, op, unary_op, register):
         try:
@@ -313,7 +402,7 @@ class Types(ModuleAnalysis):
                     node = ast.Name(node_id, ast.Load())
                 self.name_to_nodes.setdefault(node_id, set()).add(node)
 
-                former_unary_op = copy_func(unary_op)
+                former_unary_op = unary_op
 
                 # update the type to reflect container nesting
                 unary_op = lambda x: reduce(lambda t, n: ContainerType(t),
@@ -324,49 +413,51 @@ class Types(ModuleAnalysis):
                 if node not in self.result:
                     self.result[node] = new_type
             else:
-                if register and self.isargument(node):
+                # only perform inter procedural combination upon stage 0
+                if register and self.isargument(node) and self.stage == 0:
                     node_id, _ = self.node_to_id(node)
                     if node not in self.result:
                         self.result[node] = unary_op(self.result[othernode])
                     assert self.result[node], "found an alias with a type"
 
-                    parametric_type = PType(self.current[-1],
+                    parametric_type = PType(self.current,
                                             self.result[othernode])
-                    self.register(parametric_type)
+                    if self.register(parametric_type):
 
-                    def translator_generator(args, op, unary_op):
-                        ''' capture args for translator generation'''
-                        def interprocedural_type_translator(s, n):
-                            translated_othernode = ast.Name(
-                                '__fake__', ast.Load())
-                            s.result[translated_othernode] = (
-                             parametric_type.instanciate(
-                             s.current[-1], [s.result[arg] for arg in n.args]))
-                            # look for modified argument
-                            for p, effective_arg in enumerate(n.args):
-                                formal_arg = args[p]
-                                if formal_arg.id == node_id:
-                                    translated_node = effective_arg
-                                    break
-                            try:
-                                s.combine(translated_node,
-                                    translated_othernode,
-                                    op, unary_op, register=True)
-                            except NotImplementedError:
-                                pass
-                                # this may fail when the effective
-                                #parameter is an expression
-                            except UnboundLocalError:
-                                pass
-                                # this may fail when translated_node
-                                #is a default parameter
-                        return interprocedural_type_translator
-                    translator = translator_generator(
-                        self.current[-1].args.args,
-                        op, unary_op)  # deferred combination
-                    user_module = modules['__user__']
-                    current_function = user_module[self.current[-1].name]
-                    current_function.add_combiner(translator)
+                        user_module = modules['__user__']
+                        current_function = user_module[self.current.name]
+
+                        def translator_generator(args, op, unary_op):
+                            ''' capture args for translator generation'''
+                            def interprocedural_type_translator(s, n):
+                                translated_othernode = ast.Name(
+                                    '__fake__', ast.Load())
+                                s.result[translated_othernode] = (
+                                 parametric_type.instanciate(
+                                 s.current, [s.result[arg] for arg in n.args]))
+                                # look for modified argument
+                                for p, effective_arg in enumerate(n.args):
+                                    formal_arg = args[p]
+                                    if formal_arg.id == node_id:
+                                        translated_node = effective_arg
+                                        break
+                                try:
+                                    s.combine(translated_node,
+                                        translated_othernode,
+                                        op, unary_op, register=True)
+                                except NotImplementedError:
+                                    pass
+                                    # this may fail when the effective
+                                    #parameter is an expression
+                                except UnboundLocalError:
+                                    pass
+                                    # this may fail when translated_node
+                                    #is a default parameter
+                            return interprocedural_type_translator
+                        translator = translator_generator(
+                            self.current.args.args,
+                            op, unary_op)  # deferred combination
+                        current_function.add_combiner(translator)
                 else:
                     new_type = unary_op(self.result[othernode])
                     if node not in self.result:
@@ -376,14 +467,11 @@ class Types(ModuleAnalysis):
         except UnboundableRValue:
             pass
         except:
-            print ast.dump(othernode)
+            #print ast.dump(othernode)
             raise
 
-    def visit_Module(self, node):
-        [self.visit(n) for n in node.body]
-
     def visit_FunctionDef(self, node):
-        self.current.append(node)
+        self.current = node
         self.typedefs = list()
         self.name_to_nodes = {arg.id: {arg} for arg in node.args.args}
         self.yield_points = self.passmanager.gather(YieldPoints, node)
@@ -392,8 +480,17 @@ class Types(ModuleAnalysis):
             fake_node = ast.Name(k, ast.Load())
             self.name_to_nodes.update({k: {fake_node}})
             self.result[fake_node] = NamedType(v)
-        self.visit(node.args)
-        [self.visit(n) for n in node.body]
+
+        # two stages, one for inter procedural propagation
+        self.stage = 0
+        self.generic_visit(node)
+
+        # and one for backward propagation
+        # but this step is generally costly
+        if cfg.getboolean('typing', 'enable_two_steps_typing'):
+            self.stage = 1
+            self.generic_visit(node)
+
         # propagate type information through all aliases
         for name, nodes in self.name_to_nodes.iteritems():
             final_node = ast.Name("__fake__" + name, ast.Load())
@@ -403,125 +500,120 @@ class Types(ModuleAnalysis):
                 self.result[n] = self.result[final_node]
         self.current_global_declarations[node.name] = node
         self.result[node] = (Assignable(self.result[node]), self.typedefs)
-        self.current.pop()
+        for k in self.passmanager.gather(LocalDeclarations, node):
+            lazy_res = self.lazyness_analysis[k.id]
+            if lazy_res <= self.max_recompute:
+                self.result[k] = Lazy(self.result[k])
+            else:
+                self.result[k] = Assignable(self.result[k])
 
     def visit_Return(self, node):
+        self.generic_visit(node)
         if not self.yield_points:
             if node.value:
-                self.visit(node.value)
-                self.combine(self.current[-1], node.value)
+                self.combine(self.current, node.value)
             else:
-                self.result[self.current[-1]] = NamedType("none_type")
+                self.result[self.current] = NamedType("none_type")
 
     def visit_Yield(self, node):
-        self.visit(node.value)
-        self.combine(self.current[-1], node.value)
-
-    def visit_TryExcept(self, node):
-        [self.visit(n) for n in node.body]
-        [self.visit(n) for n in node.handlers]
+        self.generic_visit(node)
+        self.combine(self.current, node.value)
 
     def visit_Assign(self, node):
         self.visit(node.value)
         for t in node.targets:
             self.combine(t, node.value, register=True)
             if isinstance(t, ast.Subscript):
-                self.visit_AssignedSubscript(t)
-                for alias in self.strict_aliases[t.value].aliases:
-                    fake = ast.Subscript(alias, t.value, ast.Store())
-                    self.combine(fake, node.value, register=True)
+                if self.visit_AssignedSubscript(t):
+                    for alias in self.strict_aliases[t.value].aliases:
+                        fake = ast.Subscript(alias, t.value, ast.Store())
+                        self.combine(fake, node.value, register=True)
 
     def visit_AugAssign(self, node):
         self.visit(node.value)
         self.combine(node.target, node.value,
-            lambda x, y: ExpressionType(operator_to_lambda[type(node.op)],
+            lambda x, y: x + ExpressionType(operator_to_lambda[type(node.op)],
                 [x, y]), register=True)
         if isinstance(node.target, ast.Subscript):
-            self.visit_AssignedSubscript(node.target)
-            for alias in self.strict_aliases[node.target.value].aliases:
-                fake = ast.Subscript(alias, node.target.value, ast.Store())
-                self.combine(fake,
-                        node.value,
-                        lambda x, y: ExpressionType(
-                            operator_to_lambda[type(node.op)],
-                            [x, y]),
-                        register=True)
+            if self.visit_AssignedSubscript(node.target):
+                for alias in self.strict_aliases[node.target.value].aliases:
+                    fake = ast.Subscript(alias, node.target.value, ast.Store())
+                    self.combine(fake,
+                            node.value,
+                            lambda x, y: x + ExpressionType(
+                                operator_to_lambda[type(node.op)],
+                                [x, y]),
+                            register=True)
 
     def visit_For(self, node):
         self.visit(node.iter)
         self.combine(node.target, node.iter,
-            unary_op=lambda x: IteratorContentType(x), register=True)
-        self.visit(node.target)
-        if node.body:
-            [self.visit(n) for n in node.body]
-        if node.orelse:
-            [self.visit(n) for n in node.orelse]
+            unary_op=IteratorContentType, register=True)
+        node.body and map(self.visit, node.body)
+        node.orelse and map(self.visit, node.orelse)
 
     def visit_BoolOp(self, node):
-        [self.visit(value) for value in node.values]
+        self.generic_visit(node)
         [self.combine(node, value) for value in node.values]
 
     def visit_BinOp(self, node):
-        [self.visit(value) for value in (node.left, node.right)]
+        self.generic_visit(node)
         wl, wr = [self.result[x].isweak() for x in (node.left, node.right)]
         if (isinstance(node.op, ast.Add) and any([wl, wr])
             and not all([wl, wr])):
         # assumes the + operator always has the same operand type
-        #on left and right side
-            F = lambda x, y: x + y
+        # on left and right side
+            F = operator.add
         else:
             F = lambda x, y: ExpressionType(
                 operator_to_lambda[type(node.op)], [x, y])
 
-        self.combine(node, node.left, F)
-        self.combine(node, node.right, F)
+        fake_node = ast.Name("#", ast.Param())
+        self.combine(fake_node, node.left, F)
+        self.combine(fake_node, node.right, F)
+        self.combine(node, fake_node)
+        del self.result[fake_node]
 
     def visit_UnaryOp(self, node):
-        self.visit(node.operand)
+        self.generic_visit(node)
         f = lambda x: ExpressionType(operator_to_lambda[type(node.op)], [x])
         self.combine(node, node.operand, unary_op=f)
 
-    def visit_Lambda(self, node):
-        assert False
-
     def visit_IfExp(self, node):
-        [self.visit(n) for n in (node.body, node.orelse)]
+        self.generic_visit(node)
         [self.combine(node, n) for n in (node.body, node.orelse)]
 
     def visit_Compare(self, node):
         self.generic_visit(node)
-        self.result[node] = NamedType("bool")
+        all_compare = zip(node.ops, node.comparators)
+        for op, comp in all_compare:
+            self.combine(node, comp,
+                    unary_op=lambda x: ExpressionType(
+                        operator_to_lambda[type(op)],
+                        [self.result[node.left], x])
+                    )
 
     def visit_Call(self, node):
-        self.visit(node.func)
-        [self.visit(n) for n in node.args]
+        self.generic_visit(node)
         user_module = modules['__user__']
         for alias in self.strict_aliases[node.func].aliases:
             # handle backward type dependencies from method calls
-            if isinstance(alias, ast.Attribute):
-                if isinstance(alias.value, ast.Name):
-                    if (alias.value.id in modules and
-                        alias.attr in modules[alias.value.id]):
-                        aliased_fun = modules[alias.value.id][alias.attr]
-                        if hasattr(aliased_fun, 'combiner'):
-                            # aliased_fun.combiner(self, node)
-                            # also generate all possible aliasing combinations
-                            combinations = [filter(
-                                None,
-                                self.strict_aliases[arg].aliases.union({arg}))
-                                for arg in node.args]
-                            for new_args in itertools.product(*combinations):
-                                aliased_fun.combiner(self,
-                                        ast.Call(
-                                            node.func,
-                                            new_args,
-                                            [],
-                                            None,
-                                            None)
-                                        )
-                else:
-                    raise PythranSyntaxError(
-                        "Unknown Attribute: `{0}'".format(alias.attr), node)
+            if isinstance(alias, intrinsic.Intrinsic):
+                if hasattr(alias, 'combiner'):
+                    # also generate all possible aliasing combinations
+                    combinations = [filter(
+                        None,
+                        self.strict_aliases[arg].aliases.union({arg}))
+                        for arg in node.args]
+                    for new_args in itertools.product(*combinations):
+                        alias.combiner(self,
+                                ast.Call(
+                                    node.func,
+                                    new_args,
+                                    [],
+                                    None,
+                                    None)
+                                )
             # handle backward type dependencies from user calls
             elif isinstance(alias, ast.FunctionDef):
                 user_module[alias.name].combiner(self, node)
@@ -538,7 +630,20 @@ class Types(ModuleAnalysis):
                     self.result[n] = ReturnType(self.result[alias.func],
                         [self.result[arg] for arg in alias.args])
 
+        # recurring nightmare
+        isweak = any(self.result[n].isweak() for n in node.args + [node.func])
+        if self.stage == 0 and isweak:
+            # maybe we can get saved if we have a hint about
+            # the called function return type
+            for alias in self.strict_aliases[node.func].aliases:
+                if alias is self.current and alias in self.result:
+                    # great we have a (partial) type information
+                    self.result[node] = self.result[alias]
+                    return
+
+        # default behavior
         F = lambda f: ReturnType(f, [self.result[arg] for arg in node.args])
+        # op is used to drop previous value there
         self.combine(node, node.func, op=lambda x, y: y, unary_op=F)
 
     def visit_Num(self, node):
@@ -548,40 +653,59 @@ class Types(ModuleAnalysis):
         self.result[node] = NamedType(pytype_to_ctype_table[str])
 
     def visit_Attribute(self, node):
-        value, attr = (node.value, node.attr)
-        if value.id in modules and attr in modules[value.id]:
-            self.result[node] = (DeclType(
-                Val('{0}::{1}'.format(value.id, attr)))
-                if modules[value.id][attr].isscalar()
-                else DeclType(Val("{0}::proxy::{1}()".format(value.id, attr)))
+        def rec(w, n):
+            if isinstance(n, ast.Name):
+                return w[n.id], (n.id,)
+            elif isinstance(n, ast.Attribute):
+                r = rec(w, n.value)
+                return r[0][n.attr], r[1] + (n.attr,)
+        obj, path = rec(modules, node)
+        self.result[node] = DeclType(
+                '::'.join(path) if obj.isliteral() else
+                ('::'.join(path[:-1]) + '::proxy::' + path[-1] + '()')
                 )
-        else:
-            raise PythranSyntaxError(
-                "Unknown Attribute: `{0}'".format(node.attr), node)
+
+    def visit_Slice(self, node):
+        self.generic_visit(node)
+        self.result[node] = NamedType('core::slice')
 
     def visit_Subscript(self, node):
         self.visit(node.value)
         if metadata.get(node, metadata.Attribute):
             f = lambda t: AttributeType(node.slice.value.n, t)
+        elif isinstance(node.slice, ast.ExtSlice):
+            d = sum(int(type(dim) is ast.Index) for dim in node.slice.dims)
+            f = lambda t: reduce(lambda x, y: ContentType(x), range(d), t)
         elif isinstance(node.slice, ast.Slice):
-            f = lambda t: t
-        elif isinstance(node.slice.value, ast.Num):
+            self.visit(node.slice)
+            f = lambda x:  ExpressionType(
+                    lambda a, b: "{0}[{1}]".format(a, b),
+                    [x, self.result[node.slice]]
+                    )
+        elif isinstance(node.slice.value, ast.Num) and node.slice.value.n >= 0:
             f = lambda t: ElementType(node.slice.value.n, t)
         elif isinstance(node.slice.value, ast.Tuple):
             f = lambda t: reduce(lambda x, y: ContentType(x),
                     node.slice.value.elts, t)
         else:
-            f = lambda t: ContentType(t)
-        self.combine(node, node.value, unary_op=f)
+            self.visit(node.slice)
+            f = lambda x: ExpressionType(
+                    lambda a, b: "{0}[{1}]".format(a, b),
+                    [x, self.result[node.slice]]
+                    )
+        f and self.combine(node, node.value, unary_op=f)
 
     def visit_AssignedSubscript(self, node):
-        if not isinstance(node.slice, ast.Slice):
+        if type(node.slice) not in (ast.Slice, ast.ExtSlice):
             self.visit(node.slice)
             self.combine(node.value, node.slice,
-                unary_op=lambda t: IndexableType(t), register=True)
+                    unary_op=IndexableType, register=True)
             for alias in self.strict_aliases[node.value].aliases:
                 self.combine(alias, node.slice,
-                    unary_op=lambda t: IndexableType(t), register=True)
+                        unary_op=IndexableType, register=True)
+            return True
+        else:
+            return False
 
     def visit_Name(self, node):
         if node.id in self.name_to_nodes:
@@ -591,52 +715,47 @@ class Types(ModuleAnalysis):
             self.combine(node, self.current_global_declarations[node.id])
         elif node.id in builtin_constants:
             self.result[node] = NamedType(builtin_constants[node.id])
-        elif node.id in modules["__builtins__"]:
-            self.result[node] = NamedType("proxy::{0}".format(node.id))
         elif node.id in builtin_constructors:
-            self.result[node] = NamedType("pythonic::constructor<{0}>".format(
-                builtin_constructors[node.id]))
+            self.result[node] = ConstructorType(
+                    NamedType(builtin_constructors[node.id]))
         else:
             self.result[node] = NamedType(node.id, {Weak})
 
     def visit_List(self, node):
+        self.generic_visit(node)
         if node.elts:
-            [self.visit(elt) for elt in node.elts]
-            [self.combine(node, elt,
-                unary_op=lambda x:ListType(x)) for elt in node.elts]
+            for elt in node.elts:
+                self.combine(node, elt, unary_op=ListType)
         else:
             self.result[node] = NamedType("core::empty_list")
 
     def visit_Set(self, node):
+        self.generic_visit(node)
         if node.elts:
-            [self.visit(elt) for elt in node.elts]
-            [self.combine(node, elt,
-                unary_op=lambda x:SetType(x)) for elt in node.elts]
+            for elt in node.elts:
+                self.combine(node, elt, unary_op=SetType)
         else:
             self.result[node] = NamedType("core::empty_set")
 
     def visit_Dict(self, node):
+        self.generic_visit(node)
         if node.keys:
-            [self.visit(elt) for elt in node.keys]
-            [self.visit(elt) for elt in node.values]
-            [self.combine(
-                node,
-                key,
-                unary_op=lambda x:DictType(x, self.result[value]))
-                for key, value in zip(node.keys, node.values)]
+            for key, value in zip(node.keys, node.values):
+                self.combine(node, key,
+                        unary_op=lambda x: DictType(x, self.result[value]))
         else:
             self.result[node] = NamedType("core::empty_dict")
 
     def visit_ExceptHandler(self, node):
         if node.type and node.name:
             if not isinstance(node.type, ast.Tuple):
-                self.result[node.type] = NamedType(
-                    "core::{0}".format(node.type.id))
+                tname = NamedType('core::{0}'.format(node.type.attr))
+                self.result[node.type] = tname
                 self.combine(node.name, node.type, register=True)
-        [self.visit(n) for n in node.body]
+        map(self.visit, node.body)
 
     def visit_Tuple(self, node):
-        [self.visit(elt) for elt in node.elts]
+        self.generic_visit(node)
         try:
             types = [self.result[elt] for elt in node.elts]
             self.result[node] = TupleType(types)
@@ -644,10 +763,10 @@ class Types(ModuleAnalysis):
             pass  # not very harmonious with the combine method ...
 
     def visit_Index(self, node):
-        self.visit(node.value)
+        self.generic_visit(node)
         self.combine(node, node.value)
 
     def visit_arguments(self, node):
         for i, arg in enumerate(node.args):
             self.result[arg] = ArgumentType(i)
-        [self.visit(n) for n in node.defaults]
+        map(self.visit, node.defaults)
